#version 150

#moj_import <light.glsl>
#moj_import <fog.glsl>

#moj_import <map_data.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler0;
uniform sampler2D Sampler2;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec3 ChunkOffset;
uniform int FogShape;

uniform float GameTime;

out float vertexDistance;
out vec4 vertexColor;
out vec2 texCoord0;
out vec4 normal;

out vec3 chunkPosition;
out vec3 blockPosition;
out vec4 fractalPosition;
out vec4 baseVertexColor;

void main() {
    vec4 texCol = texture(Sampler0, UV0);

    vec3 blockPos = Position;

    if (texCol.a >= 0.6 && texCol.a < 0.7) {
        blockPos = vec3((blockPos.x - 8.0) * 16.0, (blockPos.y - 3.0) * 16.0, blockPos.z * 16.0);

        blockPos.x += sin((blockPos.y / 50.0) + (GameTime * (PI * 100.0))) * 10.0;
    }
    else if (texCol.a >= 0.5 && texCol.a < 0.6) {
        blockPos = vec3((blockPos.x / 16.0) + 8.0, (blockPos.y / 16.0) + 10.0, ((blockPos.z + 3.0) / 16.0) + (14.0 + BLOCK_OVERLAY_OFFSET));
    }
    
    vec3 pos = blockPos + ChunkOffset;
     
    if (texCol.a >= 0.8 && texCol.a < 0.9) {
        if (ModelViewMat[0].x > 0.0) {
            pos.z += REPEAT_LENGTH;
        }

        pos.z -= fract(GameTime * BLOCK_MOVE_SPEED) * REPEAT_LENGTH;
    }

    chunkPosition = ChunkOffset;
    blockPosition = Position;
    baseVertexColor = Color;
    fractalPosition = ProjMat * vec4(pos, 1.0);

    gl_Position = ProjMat * ModelViewMat * vec4(pos, 1.0);

    vertexDistance = fog_distance(ModelViewMat, pos, FogShape);
    vertexColor = Color * minecraft_sample_lightmap(Sampler2, UV2);
    texCoord0 = UV0;
    normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
}
